@@ -1,5 +1,5 @@
 /**
- * db.c : wrapper class around pmdb_t
+ * db.c : wrapper class around alpm_db_t
  *
  *  Copyright (c) 2011 Rémy Oudompheng <remy@archlinux.org>
  *
@@ -31,4 +31,4 @@ int pylist_db_to_alpmlist(PyObject *list, alpm_list_t **result);
 PyObject* pyalpm_find_grp_pkgs(PyObject* self, PyObject* args);
 PyObject* pyalpm_sync_newversion(PyObject *self, PyObject* args);
 
-#endif // _PYALPM_DB_H
+#endif
