@@ -24,157 +24,161 @@
 #include <alpm.h>
 #include <Python.h>
 #include "package.h"
+#include "handle.h"
 #include "util.h"
 
 /** Transaction callbacks */
-static PyObject *event_cb = NULL;
-static PyObject *conv_cb = NULL;
-static PyObject *progress_cb = NULL;
+extern PyObject *global_py_callbacks[N_CALLBACKS];
 
-static void pyalpm_trans_eventcb(pmtransevt_t event, void* data1, void *data2) {
+void pyalpm_eventcb(alpm_event_t event, void* data1, void *data2) {
   const char *eventstr;
   PyObject *obj1 = Py_None;
   PyObject *obj2 = Py_None;
   switch(event) {
-    case PM_TRANS_EVT_CHECKDEPS_START:
+    case ALPM_EVENT_CHECKDEPS_START:
       eventstr = "Checking dependencies";
       break;
-    case PM_TRANS_EVT_CHECKDEPS_DONE:
+    case ALPM_EVENT_CHECKDEPS_DONE:
       eventstr = "Done checking dependencies";
       break;
-    case PM_TRANS_EVT_FILECONFLICTS_START:
+    case ALPM_EVENT_FILECONFLICTS_START:
       eventstr = "Checking file conflicts";
       break;
-    case PM_TRANS_EVT_FILECONFLICTS_DONE:
+    case ALPM_EVENT_FILECONFLICTS_DONE:
       eventstr = "Done checking file conflicts";
       break;
-    case PM_TRANS_EVT_RESOLVEDEPS_START:
+    case ALPM_EVENT_RESOLVEDEPS_START:
       eventstr = "Resolving dependencies";
       break;
-    case PM_TRANS_EVT_RESOLVEDEPS_DONE:
+    case ALPM_EVENT_RESOLVEDEPS_DONE:
       eventstr = "Done resolving dependencies";
       break;
-    case PM_TRANS_EVT_INTERCONFLICTS_START:
+    case ALPM_EVENT_INTERCONFLICTS_START:
       eventstr = "Checking inter conflicts";
       break;
-    case PM_TRANS_EVT_INTERCONFLICTS_DONE:
+    case ALPM_EVENT_INTERCONFLICTS_DONE:
       eventstr = "Done checking inter conflicts";
       break;
-    case PM_TRANS_EVT_ADD_START:
+    case ALPM_EVENT_ADD_START:
       eventstr = "Adding a package";
       obj1 = pyalpm_package_from_pmpkg(data1);
       break;
-    case PM_TRANS_EVT_ADD_DONE:
+    case ALPM_EVENT_ADD_DONE:
       eventstr = "Done adding a package";
       obj1 = pyalpm_package_from_pmpkg(data1);
       if (data2) obj2 = pyalpm_package_from_pmpkg(data2);
       break;
-    case PM_TRANS_EVT_REMOVE_START:
+    case ALPM_EVENT_REMOVE_START:
       eventstr = "Remove package";
       obj1 = pyalpm_package_from_pmpkg(data1);
       break;
-    case PM_TRANS_EVT_REMOVE_DONE:
+    case ALPM_EVENT_REMOVE_DONE:
       eventstr = "Done removing package";
       obj1 = pyalpm_package_from_pmpkg(data1);
       break;
-    case PM_TRANS_EVT_UPGRADE_START:
+    case ALPM_EVENT_UPGRADE_START:
       eventstr = "Upgrading a package";
       obj1 = pyalpm_package_from_pmpkg(data1);
       obj2 = pyalpm_package_from_pmpkg(data2);
       break;
-    case PM_TRANS_EVT_UPGRADE_DONE:
+    case ALPM_EVENT_UPGRADE_DONE:
       eventstr = "Done upgrading a package";
       obj1 = pyalpm_package_from_pmpkg(data1);
       obj2 = pyalpm_package_from_pmpkg(data2);
       break;
-    case PM_TRANS_EVT_INTEGRITY_START:
+    case ALPM_EVENT_INTEGRITY_START:
       eventstr = "Checking integrity";
       break;
-    case PM_TRANS_EVT_INTEGRITY_DONE:
+    case ALPM_EVENT_INTEGRITY_DONE:
       eventstr = "Done checking integrity";
       break;
-    case PM_TRANS_EVT_DELTA_INTEGRITY_START:
-    case PM_TRANS_EVT_DELTA_INTEGRITY_DONE:
-    case PM_TRANS_EVT_DELTA_PATCHES_START:
-    case PM_TRANS_EVT_DELTA_PATCHES_DONE:
-    case PM_TRANS_EVT_DELTA_PATCH_START:
+    case ALPM_EVENT_DELTA_INTEGRITY_START:
+    case ALPM_EVENT_DELTA_INTEGRITY_DONE:
+    case ALPM_EVENT_DELTA_PATCHES_START:
+    case ALPM_EVENT_DELTA_PATCHES_DONE:
+    case ALPM_EVENT_DELTA_PATCH_START:
       /* info here */
-    case PM_TRANS_EVT_DELTA_PATCH_DONE:
-    case PM_TRANS_EVT_DELTA_PATCH_FAILED:
-    case PM_TRANS_EVT_SCRIPTLET_INFO:
+    case ALPM_EVENT_DELTA_PATCH_DONE:
+    case ALPM_EVENT_DELTA_PATCH_FAILED:
+    case ALPM_EVENT_SCRIPTLET_INFO:
       /* info here */
-    case PM_TRANS_EVT_RETRIEVE_START:
+    case ALPM_EVENT_RETRIEVE_START:
       /* info here */
       eventstr = "event not implemented";
       break;
-    case PM_TRANS_EVT_DISKSPACE_START:
+    case ALPM_EVENT_DISKSPACE_START:
       eventstr = "Checking disk space";
       break;
-    case PM_TRANS_EVT_DISKSPACE_DONE:
+    case ALPM_EVENT_DISKSPACE_DONE:
       eventstr = "Done checking disk space";
       break;
     default:
       eventstr = "unknown event";
   }
-  PyObject *result = NULL;
-  result = PyObject_CallFunction(event_cb, "is(NN)",
-      event, eventstr, obj1, obj2);
-  if (PyErr_Occurred()) PyErr_Print();
-  Py_CLEAR(result);
+  {
+    PyObject *result = NULL;
+    if (global_py_callbacks[CB_PROGRESS]) {
+      result = PyObject_CallFunction(global_py_callbacks[CB_EVENT], "is(NN)",
+          event, eventstr, obj1, obj2);
+    } else {
+      PyErr_SetString(PyExc_RuntimeError, "event callback was called but it's not set!");
+    }
+    if (PyErr_Occurred()) PyErr_Print();
+    Py_CLEAR(result);
+  }
 }
 
-static void pyalpm_trans_convcb(pmtransconv_t question,
+void pyalpm_questioncb(alpm_question_t question,
         void* data1, void *data2, void* data3, int* retcode) {
 }
 
-static void pyalpm_trans_progresscb(pmtransprog_t op,
+void pyalpm_progresscb(alpm_progress_t op,
         const char* target_name, int percentage, size_t n_targets, size_t cur_target) {
   PyObject *result = NULL;
-  if (progress_cb) {
-    result = PyObject_CallFunction(progress_cb, "sinn",
+  if (global_py_callbacks[CB_PROGRESS]) {
+    result = PyObject_CallFunction(global_py_callbacks[CB_PROGRESS], "sinn",
       target_name, percentage, n_targets, cur_target);
   } else {
     PyErr_SetString(PyExc_RuntimeError, "progress callback was called but it's not set!");
   }
   if (PyErr_Occurred()) {
     PyErr_Print();
-    alpm_trans_interrupt();
+    /* alpm_trans_interrupt(handle); */
   }
   Py_CLEAR(result);
 }
 
 /** Transaction info translation */
 static PyObject* pyobject_from_pmdepmissing(void *item) {
-  pmdepmissing_t* miss = (pmdepmissing_t*)item;
-  char* needed = alpm_dep_compute_string(alpm_miss_get_dep(miss));
+  alpm_depmissing_t* miss = (alpm_depmissing_t*)item;
+  char* needed = alpm_dep_compute_string(miss->depend);
   PyObject *result = Py_BuildValue("(sss)",
-      alpm_miss_get_target(miss),
+      miss->target,
       needed,
-      alpm_miss_get_causingpkg(miss));
+      miss->causingpkg);
   free(needed);
   return result;
 }
 
 static PyObject* pyobject_from_pmconflict(void *item) {
-  pmconflict_t* conflict = (pmconflict_t*)item;
+  alpm_conflict_t* conflict = (alpm_conflict_t*)item;
   return Py_BuildValue("(sss)",
-      alpm_conflict_get_package1(conflict),
-      alpm_conflict_get_package2(conflict),
-      alpm_conflict_get_reason(conflict));
+      conflict->package1,
+      conflict->package2,
+      conflict->reason);
 }
 
 static PyObject* pyobject_from_pmfileconflict(void *item) {
-  pmfileconflict_t* conflict = (pmfileconflict_t*)item;
-  const char *target = alpm_fileconflict_get_target(conflict);
-  const char *filename = alpm_fileconflict_get_file(conflict);
-  switch(alpm_fileconflict_get_type(conflict)) {
-  case PM_FILECONFLICT_TARGET:
-    return Py_BuildValue("(sss)", target, filename,
-      alpm_fileconflict_get_ctarget(conflict));
-  case PM_FILECONFLICT_FILESYSTEM:
+  alpm_fileconflict_t* conflict = (alpm_fileconflict_t*)item;
+  const char *target = conflict->target;
+  const char *filename = conflict->file;
+  switch(conflict->type) {
+  case ALPM_FILECONFLICT_TARGET:
+    return Py_BuildValue("(sss)", target, filename, conflict->ctarget);
+  case ALPM_FILECONFLICT_FILESYSTEM:
     return Py_BuildValue("(ssO)", target, filename, Py_None);
   default:
-    PyErr_SetString(PyExc_RuntimeError, "invalid type for pmfileconflict_t object");
+    PyErr_Format(PyExc_RuntimeError, "invalid type %d for alpm_fileconflict_t object", conflict->type);
     return NULL;
   }
 }
@@ -204,10 +208,13 @@ const char* flagnames[19] = {
 
 static PyObject *pyalpm_trans_get_flags(PyObject *self, void *closure)
 {
-  int flags = alpm_trans_get_flags();
-  if (flags == -1) RET_ERR("no transaction defined", NULL);
-  PyObject *result = PyDict_New();
-  for (int i = 0; i < 18; i++) {
+  PyObject *result;
+  alpm_handle_t *handle = ALPM_HANDLE(self);
+  int flags = alpm_trans_get_flags(handle);
+  int i;
+  if (flags == -1) RET_ERR("no transaction defined", alpm_errno(handle), NULL);
+  result = PyDict_New();
+  for (i = 0; i < 18; i++) {
     if(flagnames[i])
       PyDict_SetItemString(result, flagnames[i], flags & (1 << i) ? Py_True : Py_False);
   }
@@ -216,21 +223,25 @@ static PyObject *pyalpm_trans_get_flags(PyObject *self, void *closure)
 
 static PyObject *pyalpm_trans_get_add(PyObject *self, void *closure)
 {
+  alpm_handle_t *handle = ALPM_HANDLE(self);
+  alpm_list_t *to_add;
   /* sanity check */
-  int flags = alpm_trans_get_flags();
-  if (flags == -1) RET_ERR("no transaction defined", NULL);
+  int flags = alpm_trans_get_flags(handle);
+  if (flags == -1) RET_ERR("no transaction defined", alpm_errno(handle), NULL);
 
-  alpm_list_t *to_add = alpm_trans_get_add();
+  to_add = alpm_trans_get_add(handle);
   return alpmlist_to_pylist(to_add, pyalpm_package_from_pmpkg);
 }
 
 static PyObject *pyalpm_trans_get_remove(PyObject *self, void *closure)
 {
+  alpm_handle_t *handle = ALPM_HANDLE(self);
+  alpm_list_t *to_remove;
   /* sanity check */
-  int flags = alpm_trans_get_flags();
-  if (flags == -1) RET_ERR("no transaction defined", NULL);
+  int flags = alpm_trans_get_flags(handle);
+  if (flags == -1) RET_ERR("no transaction defined", alpm_errno(handle), NULL);
 
-  alpm_list_t *to_remove = alpm_trans_get_remove();
+  to_remove = alpm_trans_get_remove(handle);
   return alpmlist_to_pylist(to_remove, pyalpm_package_from_pmpkg);
 }
 
@@ -243,140 +254,152 @@ static PyObject *pyalpm_trans_get_remove(PyObject *self, void *closure)
   array[14], array[15], array[16], \
   array[17]
 
-static PyObject* pyalpm_trans_init(PyObject *self, PyObject *args, PyObject *kwargs) {
-  const char* keywords[] = {
-    INDEX_FLAGS(flagnames),
-    "event_callback",
-    "conv_callback",
-    "progress_callback", NULL };
+/** Initializes a transaction
+ * @param self a Handle object
+ * ...
+ * @return a Transaction object with the same underlying object
+ */
+PyObject* pyalpm_trans_init(PyObject *self, PyObject *args, PyObject *kwargs) {
+  alpm_handle_t *handle = ALPM_HANDLE(self);
+  PyObject *result;
+  const char* keywords[] = { INDEX_FLAGS(flagnames), NULL };
   char flags[18] = "\0\0\0\0\0" /* 5 */ "\0\0\0\0\0" /* 10 */ "\0\0\0\0\0" /* 15 */ "\0\0\0";
-  Py_CLEAR(event_cb);
-  Py_CLEAR(conv_cb);
-  Py_CLEAR(progress_cb);
 
   /* check all arguments */
   if (!PyArg_ParseTupleAndKeywords(args, kwargs,
-        "|iiiiiiiiiiiiiiiiOOO", keywords,
-        INDEX_FLAGS(&flags), &event_cb, &conv_cb, &progress_cb)) {
+        "|bbbbbbbbbbbbbbbbOOO", (char**)keywords,
+        INDEX_FLAGS(&flags))) {
     return NULL;
   }
-  /* build arguments */
-  int flag_int = 0;
-  for (int i = 0; i < 18; i++) {
-    if (flags[i]) flag_int |= 1 << i;
-  }
-  if (event_cb) {
-    if (!PyCallable_Check(event_cb)) {
-      event_cb = NULL;
-      PyErr_SetString(PyExc_TypeError, "event_callback is not callable!");
-    } else {
-      Py_INCREF(event_cb);
-    }
-  }
-  if (conv_cb) {
-    if (!PyCallable_Check(conv_cb)) {
-      conv_cb = NULL;
-      PyErr_SetString(PyExc_TypeError, "conv_callback is not callable!");
-    } else {
-      Py_INCREF(conv_cb);
-    }
-  }
-  if (progress_cb) {
-    if(!PyCallable_Check(progress_cb)) {
-      progress_cb = NULL;
-      PyErr_SetString(PyExc_TypeError, "progress_callback is not callable!");
-    } else {
-      Py_INCREF(progress_cb);
-    }
-  }
 
   /* run alpm_trans_init() */
-  int ret = alpm_trans_init(flag_int
-      , event_cb ? pyalpm_trans_eventcb : NULL
-      , conv_cb ? pyalpm_trans_convcb : NULL
-      , progress_cb ? pyalpm_trans_progresscb : NULL);
-  if (ret == -1) {
-    RET_ERR("transaction could not be initialized", NULL);
+  {
+    alpm_transflag_t flag_int = 0;
+    int i, ret;
+    for (i = 0; i < 18; i++) {
+      if (flags[i]) flag_int |= 1 << i;
+    }
+    ret = alpm_trans_init(handle, flag_int);
+    if (ret == -1) {
+      RET_ERR("transaction could not be initialized", alpm_errno(handle), NULL);
+    }
   }
-  Py_RETURN_NONE;
+  result = pyalpm_transaction_from_pmhandle(handle);
+  return result;
 }
 
 static PyObject* pyalpm_trans_prepare(PyObject *self, PyObject *args) {
+  alpm_handle_t *handle = ALPM_HANDLE(self);
   alpm_list_t *data;
 
-  int ret = alpm_trans_prepare(&data);
+  int ret = alpm_trans_prepare(handle, &data);
   if (ret == -1) {
     /* return the list of package conflicts in the exception */
     PyObject *info = alpmlist_to_pylist(data, pyobject_from_pmdepmissing);
     if (!info) return NULL;
-    RET_ERR_DATA("transaction preparation failed", info, NULL);
+    RET_ERR_DATA("transaction preparation failed", alpm_errno(handle), info, NULL);
   }
 
   Py_RETURN_NONE;
 }
 
 static PyObject* pyalpm_trans_commit(PyObject *self, PyObject *args) {
+  alpm_handle_t *handle = ALPM_HANDLE(self);
   alpm_list_t *data = NULL;
+  int ret;
+  enum _alpm_errno_t err;
+  PyObject *err_info = NULL;
 
-  int ret = alpm_trans_commit(&data);
-  if (ret == -1) {
-    /* return the list of file conflicts in the exception */
-    PyObject *info = alpmlist_to_pylist(data, pyobject_from_pmfileconflict);
-    if (!info) return NULL;
-    RET_ERR_DATA("transaction failed", info, NULL);
+  ret = alpm_trans_commit(handle, &data);
+  if (ret == 0) Py_RETURN_NONE;
+  if (ret != -1) {
+    PyErr_Format(PyExc_RuntimeError,
+        "unexpected return value %d from alpm_trans_commit()", ret);
+    return NULL;
   }
 
-  Py_RETURN_NONE;
+  err = alpm_errno(handle);
+  switch(err) {
+    case ALPM_ERR_FILE_CONFLICTS:
+      /* return the list of file conflicts in the exception */
+      err_info = alpmlist_to_pylist(data, pyobject_from_pmfileconflict);
+      break;
+    case ALPM_ERR_PKG_INVALID:
+    case ALPM_ERR_PKG_INVALID_CHECKSUM:
+    case ALPM_ERR_PKG_INVALID_SIG:
+    case ALPM_ERR_DLT_INVALID:
+      err_info = alpmlist_to_pylist(data, pyobject_from_string);
+      break;
+    default:
+      break;
+  }
+  if (err_info)
+    RET_ERR_DATA("transaction failed", err, err_info, NULL);
+  else
+    RET_ERR("transaction failed", err, NULL);
 }
 
 static PyObject* pyalpm_trans_interrupt(PyObject *self, PyObject *args) {
-  int ret = alpm_trans_interrupt();
-  if (ret == -1) RET_ERR("unable to interrupt transaction", NULL);
+  alpm_handle_t *handle = ALPM_HANDLE(self);
+  int ret = alpm_trans_interrupt(handle);
+  if (ret == -1) RET_ERR("unable to interrupt transaction", alpm_errno(handle), NULL);
   Py_RETURN_NONE;
 }
 
 PyObject* pyalpm_trans_release(PyObject *self, PyObject *args) {
-  int ret = alpm_trans_release();
-  if (ret == -1) RET_ERR("unable to release transaction", NULL);
+  alpm_handle_t *handle = ALPM_HANDLE(self);
+  int ret = alpm_trans_release(handle);
+  if (ret == -1) RET_ERR("unable to release transaction", alpm_errno(handle), NULL);
   Py_RETURN_NONE;
 }
 
 /** Transaction contents */
 static PyObject* pyalpm_trans_add_pkg(PyObject *self, PyObject *args) {
+  alpm_handle_t *handle = ALPM_HANDLE(self);
+  alpm_pkg_t *pmpkg;
   PyObject *pkg;
+  int ret;
+
   if (!PyArg_ParseTuple(args, "O!", &AlpmPackageType, &pkg)) {
     return NULL;
   }
 
-  pmpkg_t *pmpkg = pmpkg_from_pyalpm_pkg(pkg);
-  int ret = alpm_add_pkg(pmpkg);
-  if (ret == -1) RET_ERR("unable to update transaction", NULL);
+  pmpkg = pmpkg_from_pyalpm_pkg(pkg);
+  ret = alpm_add_pkg(handle, pmpkg);
+  if (ret == -1) RET_ERR("unable to update transaction", alpm_errno(handle), NULL);
   /* alpm_add_pkg eats the reference to pkg */
   pyalpm_pkg_unref(pkg);
   Py_RETURN_NONE;
 }
 
 static PyObject* pyalpm_trans_remove_pkg(PyObject *self, PyObject *args) {
+  alpm_handle_t *handle = ALPM_HANDLE(self);
   PyObject *pkg;
+  alpm_pkg_t *pmpkg;
+  int ret;
+
   if (!PyArg_ParseTuple(args, "O!", &AlpmPackageType, &pkg)) {
     return NULL;
   }
 
-  pmpkg_t *pmpkg = pmpkg_from_pyalpm_pkg(pkg);
-  int ret = alpm_remove_pkg(pmpkg);
-  if (ret == -1) RET_ERR("unable to update transaction", NULL);
+  pmpkg = pmpkg_from_pyalpm_pkg(pkg);
+  ret = alpm_remove_pkg(handle, pmpkg);
+  if (ret == -1) RET_ERR("unable to update transaction", alpm_errno(handle), NULL);
   Py_RETURN_NONE;
 }
 
 static PyObject* pyalpm_trans_sysupgrade(PyObject *self, PyObject *args, PyObject *kwargs) {
+  alpm_handle_t *handle = ALPM_HANDLE(self);
   char* keyword[] = {"downgrade", NULL};
   PyObject *downgrade;
+  int do_downgrade, ret;
+
   if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!", keyword, &PyBool_Type, &downgrade))
     return NULL;
 
-  int do_downgrade = (downgrade == Py_True) ? 1 : 0;
-  int ret = alpm_sync_sysupgrade(do_downgrade);
-  if (ret == -1) RET_ERR("unable to update transaction", NULL);
+  do_downgrade = (downgrade == Py_True) ? 1 : 0;
+  ret = alpm_sync_sysupgrade(handle, do_downgrade);
+  if (ret == -1) RET_ERR("unable to update transaction", alpm_errno(handle), NULL);
   Py_RETURN_NONE;
 }
 
@@ -392,19 +415,6 @@ static struct PyGetSetDef pyalpm_trans_getset[] = {
 
 static struct PyMethodDef pyalpm_trans_methods[] = {
   /* Execution flow */
-  {"init",    (PyCFunction)pyalpm_trans_init, METH_VARARGS | METH_KEYWORDS,
-    "Initializes a transaction.\n"
-    "Arguments:\n"
-    "  nodeps, force, nosave, nodepversion, cascade, recurse,\n"
-    "  dbonly, alldeps, downloadonly, noscriptlet, noconflicts,\n"
-    "  needed, allexplicit, inneeded, recurseall, nolock\n"
-    "    -- the transaction options (booleans)\n"
-    "  event_callback -- a function called when an event occurs\n"
-    "    -- args: (event ID, event string, (object 1, object 2))\n"
-    "  conv_callback -- a function called to get user input\n"
-    "  progress_callback -- a function called to indicate progress\n"
-    "    -- args: (target name, percentage, number of targets, target number)\n"
-  },
   {"prepare", pyalpm_trans_prepare,    METH_NOARGS, "prepare" },
   {"commit",  pyalpm_trans_commit,     METH_NOARGS, "commit" },
   {"interrupt", pyalpm_trans_interrupt,METH_NOARGS,  "Interrupt the transaction." },
@@ -422,51 +432,38 @@ static struct PyMethodDef pyalpm_trans_methods[] = {
   { NULL }
 };
 
+/* The Transaction object have the same underlying C structure
+ * as the Handle objects. Only the method table changes.
+ */
 static PyTypeObject AlpmTransactionType = {
   PyVarObject_HEAD_INIT(NULL, 0)
-  "alpm.TransactionClass",    /*tp_name*/
-  0,                   /*tp_basicsize*/
-  0,                   /*tp_itemsize*/
-  0,                   /*tp_dealloc*/
-  0,                   /*tp_print*/
-  0,                   /*tp_getattr*/
-  0,                   /*tp_setattr*/
-  NULL,                /*tp_reserved*/
-  0,                   /*tp_repr*/
-  0,                   /*tp_as_number*/
-  0,                   /*tp_as_sequence*/
-  0,                   /*tp_as_mapping*/
-  0,                   /*tp_hash */
-  0,                   /*tp_call*/
-  0,                   /*tp_str*/
-  0,                   /*tp_getattro*/
-  0,                   /*tp_setattro*/
-  0,                   /*tp_as_buffer*/
-  Py_TPFLAGS_DEFAULT, /*tp_flags*/
-  "This class is the main interface to get/set libalpm options",
-                      /* tp_doc */
-  0,                  /* tp_traverse */
-  0,                  /* tp_clear */
-  0,                  /* tp_richcompare */
-  0,                  /* tp_weaklistoffset */
-  0,                  /* tp_iter */
-  0,                  /* tp_iternext */
-  pyalpm_trans_methods, /* tp_methods */
-  0,                    /* tp_members */
-  pyalpm_trans_getset,  /* tp_getset */
+  "alpm.Transaction",    /*tp_name*/
+  sizeof(AlpmHandle),  /*tp_basicsize*/
+  .tp_flags = Py_TPFLAGS_DEFAULT,
+  .tp_doc = "This class is the main interface to get/set libalpm options",
+  .tp_methods = pyalpm_trans_methods,
+  .tp_getset = pyalpm_trans_getset,
 };
 
+PyObject *pyalpm_transaction_from_pmhandle(void* data) {
+  alpm_handle_t *handle = (alpm_handle_t*)data;
+  AlpmHandle *self;
+  self = (AlpmHandle*)AlpmTransactionType.tp_alloc(&AlpmTransactionType, 0);
+  if (self == NULL) {
+    PyErr_SetString(PyExc_RuntimeError, "unable to create pyalpm.Transaction object");
+    return NULL;
+  }
+
+  self->c_data = handle;
+  return (PyObject *)self;
+}
+
 /* Initialization */
 int init_pyalpm_transaction(PyObject *module) {
   if (PyType_Ready(&AlpmTransactionType) < 0)
     return -1;
   Py_INCREF(&AlpmTransactionType);
-  PyModule_AddObject(module, "TransactionClass", (PyObject*)(&AlpmTransactionType));
-
-  // the static instance
-  PyObject *the_transaction = (PyObject*)AlpmTransactionType.tp_alloc(&AlpmTransactionType, 0);
-  PyModule_AddObject(module, "transaction", the_transaction);
-  Py_INCREF(the_transaction);
+  PyModule_AddObject(module, "Transaction", (PyObject*)(&AlpmTransactionType));
   return 0;
 }
 
