@@ -37,7 +37,7 @@ static PyObject* pyalpm_error_str(PyObject* exception) {
   PyObject* args = PyObject_GetAttrString(exception, "args");
   PyObject* result;
   const char* errstring;
-  int errcode;
+  enum _alpm_errno_t errcode;
   PyObject *data;
 
   int handle_format;
@@ -77,8 +77,6 @@ void init_pyalpm_error(PyObject* module) {
   Py_INCREF(alpm_error);
 }
 
-static unsigned short init = 0;
-
 /** Python lists and libalpm lists */
 
 /** Converts a Python list of strings to an alpm_list_t linked list.
@@ -160,18 +158,17 @@ const char format_regex[] = "^%"
  */
 ssize_t printf_to_pytype_format(char *dest, const char *format, size_t len) {
   regex_t regex;
-  int ret;
-  ret = regcomp(&regex, format_regex, REG_EXTENDED);
-  if (ret != 0) {
+  regmatch_t match;
+  int n_vars = 0;
+  const char *p;
+
+  if (regcomp(&regex, format_regex, REG_EXTENDED) != 0) {
     puts("Error in regex compilation !\n");
     return -1;
   }
 
-  regmatch_t match;
-  int n_vars = 0;
-
-  const char *p;
   for (p = format; *p; p++) {
+    const char* spec;
     if(*p != '%')
       continue;
     if(p[1] == '%') {
@@ -182,7 +179,7 @@ ssize_t printf_to_pytype_format(char *dest, const char *format, size_t len) {
       /* regex does not match */
       continue;
 
-    const char* spec = p + match.rm_eo;
+    spec = p + match.rm_eo;
     switch(spec[-1]) {
     case 'd':
     case 'i':
@@ -249,23 +246,4 @@ ssize_t printf_to_pytype_format(char *dest, const char *format, size_t len) {
   return n_vars;
 }
 
-/** Initialization */
-
-void set_init(unsigned short i) {
-  init = i;
-}
-
-unsigned short check_init(void)
-{
-  switch(init)
-  {
-    case 0:
-      return 0;
-    case 1:
-      return 1;
-   default:
-      return -1;
-  }
-}
-
 /* vim: set ts=2 sw=2 et: */
